10#ifndef IMPACTX_CHRACC_H
11#define IMPACTX_CHRACC_H
42 static constexpr auto type =
"ChrAcc";
87 using BeamOptic::operator();
101 Alignment::compute_constants(refpart);
130 template<
typename T_Real=amrex::ParticleReal,
typename T_IdCpu=u
int64_t>
156 T_Real
const pti_tot2 = powi<2>(pti_tot);
157 T_Real
const ptf_tot2 = powi<2>(ptf_tot);
160 auto const mask = pti_tot2 <= 1_prt || ptf_tot2 <= 1_prt;
163 T_Real
const pzi_tot =
sqrt(powi<2>(pti_tot) - 1_prt);
164 T_Real
const pzf_tot =
sqrt(powi<2>(ptf_tot) - 1_prt);
168 T_Real
const numer = -ptf_tot + pzf_tot;
169 T_Real
const denom = -pti_tot + pzi_tot;
184 xout = cos_theta * x + sin_theta /
m_alpha * px;
185 pxout = -
m_alpha * sin_theta * x + cos_theta * px;
187 yout = cos_theta * y + sin_theta /
m_alpha * py;
188 pyout = -
m_alpha * sin_theta * y + cos_theta * py;
191 tout =
t + (pzf_tot - pzf_ref - pzi_tot + pzi_ref) /
m_ez;
192 tout += (1_prt/pzi_tot - 1_prt/pzf_tot)
204 x = cos_theta * xout + sin_theta * yout;
205 pxout = cos_theta * px + sin_theta * py;
207 y = -sin_theta * xout + cos_theta * yout;
208 pyout = -sin_theta * px + cos_theta * py;
254 refpart.pt = pt -
m_ez*slice_ds;
261 refpart.t =
t + (bgf - bgi)/
m_ez;
264 refpart.x = x + slice_ds*px/bgi;
265 refpart.y = y + slice_ds*py/bgi;
266 refpart.z = z + slice_ds*pz/bgi;
269 refpart.px = px*bgf/bgi;
270 refpart.py = py*bgf/bgi;
271 refpart.pz = pz*bgf/bgi;
274 refpart.s =
s + slice_ds;
278 using LinearTransport::operator();
360 std::log((pzf_ref - ptf_ref) / (pzi_ref - pti_ref)) /
m_ez;
367 R(5,6) = bgi * (ptf_ref / pzf_ref - pti_ref / pzi_ref) /
m_ez;
371 if (alpha == 0.0_prt)
375 R(1,2) = bgi * drift_log;
377 R(3,4) = bgi * drift_log;
391 R(1,1) = c2; R(1,2) = bgi * cs / alpha;
392 R(1,3) = cs; R(1,4) = bgi * s2 / alpha;
393 R(2,1) = -alpha * cs / bgf; R(2,2) = rbg * c2;
394 R(2,3) = -alpha * s2 / bgf; R(2,4) = rbg * cs;
395 R(3,1) = -cs; R(3,2) = -bgi * s2 / alpha;
396 R(3,3) = c2; R(3,4) = bgi * cs / alpha;
397 R(4,1) = alpha * s2 / bgf; R(4,2) = -rbg * cs;
398 R(4,3) = -alpha * cs / bgf; R(4,4) = rbg * c2;
#define AMREX_FORCE_INLINE
#define AMREX_GPU_HOST_DEVICE
#define IMPACTX_PUSH_EXTERN_TEMPLATE(ElementType)
Definition PushAll.H:78
T_ParticleType ParticleType
amrex_particle_real ParticleReal
constexpr T powi(T x) noexcept
__host__ __device__ std::pair< double, double > sincos(double x)
__host__ __device__ GpuComplex< T > log(const GpuComplex< T > &a_z) noexcept
__host__ __device__ GpuComplex< T > sqrt(const GpuComplex< T > &a_z) noexcept
@ s
fixed s as the independent variable
Definition ImpactXParticleContainer.H:37
@ t
fixed t as the independent variable
Definition ImpactXParticleContainer.H:38
amrex::SmallMatrix< amrex::ParticleReal, 6, 6, amrex::Order::F, 1 > Map6x6
Definition CovarianceMatrix.H:20
__host__ __device__ void make_invalid() const noexcept
static constexpr __host__ __device__ SmallMatrix< T, NRows, NCols, ORDER, StartIndex > Identity() noexcept
Definition ReferenceParticle.H:33
amrex::ParticleReal pt
energy, normalized by rest energy
Definition ReferenceParticle.H:42
Definition ChrUniformAcc.H:41
amrex::ParticleReal m_alpha
Definition ChrUniformAcc.H:413
amrex::ParticleReal m_pti_ref
Definition ChrUniformAcc.H:412
AMREX_GPU_HOST AMREX_FORCE_INLINE Map6x6 transport_map(RefPart const &AMREX_RESTRICT refpart) const
Definition ChrUniformAcc.H:338
void compute_constants(RefPart const &refpart)
Definition ChrUniformAcc.H:96
amrex::ParticleReal m_ez
Definition ChrUniformAcc.H:405
amrex::ParticleReal m_bgi
Definition ChrUniformAcc.H:412
void reverse()
Definition ChrUniformAcc.H:84
amrex::ParticleReal m_ptf_ref
m_ds / nslice();
Definition ChrUniformAcc.H:412
amrex::ParticleReal m_alpha_iez
Definition ChrUniformAcc.H:413
AMREX_GPU_HOST_DEVICE AMREX_FORCE_INLINE void operator()(T_Real &AMREX_RESTRICT x, T_Real &AMREX_RESTRICT y, T_Real &AMREX_RESTRICT t, T_Real &AMREX_RESTRICT px, T_Real &AMREX_RESTRICT py, T_Real &AMREX_RESTRICT pt, T_IdCpu &AMREX_RESTRICT idcpu, RefPart const &AMREX_RESTRICT refpart) const
Definition ChrUniformAcc.H:132
amrex::ParticleReal m_slice_ds
magnetic field strength in 1/m
Definition ChrUniformAcc.H:411
ChrAcc(amrex::ParticleReal ds, amrex::ParticleReal ez, amrex::ParticleReal bz, amrex::ParticleReal dx=DEFAULT_dx, amrex::ParticleReal dy=DEFAULT_dy, amrex::ParticleReal rotation_degree=DEFAULT_rotation_degree, amrex::ParticleReal aperture_x=DEFAULT_aperture_x, amrex::ParticleReal aperture_y=DEFAULT_aperture_y, int nslice=DEFAULT_nslice, std::optional< std::string > name=DEFAULT_name)
Definition ChrUniformAcc.H:63
static constexpr auto type
Definition ChrUniformAcc.H:42
ImpactXParticleContainer::ParticleType PType
Definition ChrUniformAcc.H:43
amrex::ParticleReal m_bgf
Definition ChrUniformAcc.H:412
amrex::ParticleReal m_bz
electric field strength in 1/m
Definition ChrUniformAcc.H:406
Definition alignment.H:29
AMREX_GPU_HOST_DEVICE AMREX_FORCE_INLINE amrex::ParticleReal dy() const
Definition alignment.H:193
static constexpr amrex::ParticleReal DEFAULT_dy
Definition alignment.H:34
static constexpr amrex::ParticleReal DEFAULT_dx
Definition alignment.H:33
AMREX_GPU_HOST_DEVICE AMREX_FORCE_INLINE amrex::ParticleReal dx() const
Definition alignment.H:183
AMREX_GPU_HOST AMREX_FORCE_INLINE Map6x6 rotate_aligned_map(Map6x6 const &R) const
Definition alignment.H:267
Alignment(amrex::ParticleReal dx, amrex::ParticleReal dy, amrex::ParticleReal rotation_degree)
Definition alignment.H:43
static constexpr amrex::ParticleReal DEFAULT_rotation_degree
Definition alignment.H:35
Definition beamoptic.H:567
Definition lineartransport.H:50
static constexpr std::nullopt_t DEFAULT_name
Definition named.H:30
AMREX_GPU_HOST Named(std::optional< std::string > name)
Definition named.H:59
AMREX_FORCE_INLINE std::string name() const
Definition named.H:124
Definition nofinalize.H:22
Definition pipeaperture.H:26
static constexpr amrex::ParticleReal DEFAULT_aperture_x
Definition pipeaperture.H:27
AMREX_GPU_HOST_DEVICE AMREX_FORCE_INLINE amrex::ParticleReal aperture_x() const
Definition pipeaperture.H:93
AMREX_GPU_HOST_DEVICE AMREX_FORCE_INLINE amrex::ParticleReal aperture_y() const
Definition pipeaperture.H:104
static constexpr amrex::ParticleReal DEFAULT_aperture_y
Definition pipeaperture.H:28
PipeAperture(amrex::ParticleReal aperture_x, amrex::ParticleReal aperture_y)
Definition pipeaperture.H:35
static constexpr int DEFAULT_nslice
Definition thick.H:25
Thick(amrex::ParticleReal ds, int nslice)
Definition thick.H:32
amrex::ParticleReal m_ds
Definition thick.H:70
AMREX_GPU_HOST_DEVICE AMREX_FORCE_INLINE amrex::ParticleReal ds() const
Definition thick.H:55
AMREX_GPU_HOST_DEVICE AMREX_FORCE_INLINE int nslice() const
Definition thick.H:45